class S3Checks:
    S3_CHECK_AT_REST_ENCRYPTION = "S3 buckets MUST be configured for encryption at rest using a KMS key with default SSE-S3 encryption at a minimum"
    S3_CHECK_IN_TRANSIT_ENCRYPTION = "S3 buckets MUST be configured for in-transit encryption by enabling Secure Transport"
    S3_PUBLIC_ACL_ACCESS = "S3 bucket ACLs MUST NOT allow public write or full-control access"
    S3_PUBLIC_POLICY_ACCESS = "S3 bucket policies MUST NOT allow public write or full-control access"
    S3_VERSIONING = "S3 Bucket Version SHOULD be enabled"
    S3_CHECK_TAGS = "MUST be tagged in accordance with tagging standards"

class S3MethodAssociations:
    S3_CHECK_AT_REST_ENCRYPTION = "check_encryption_at_rest"
    S3_CHECK_IN_TRANSIT_ENCRYPTION = "check_in_transit_encryption"
    S3_PUBLIC_ACL_ACCESS = "check_s3_acl"
    S3_PUBLIC_POLICY_ACCESS = "check_s3_policy"
    S3_VERSIONING = "check_s3_versioning"
    S3_CHECK_TAGS = "check_bucket_tags"

security_checks = {
    "S3_CHECK_AT_REST_ENCRYPTION": {
        "method_name": S3MethodAssociations.S3_CHECK_AT_REST_ENCRYPTION,
        "check_description": S3Checks.S3_CHECK_AT_REST_ENCRYPTION
    },
    "S3_CHECK_IN_TRANSIT_ENCRYPTION": {
        "method_name": S3MethodAssociations.S3_CHECK_IN_TRANSIT_ENCRYPTION,
        "check_description": S3Checks.S3_CHECK_IN_TRANSIT_ENCRYPTION
    },
    "S3_PUBLIC_ACL_ACCESS": {
        "method_name": S3MethodAssociations.S3_PUBLIC_ACL_ACCESS,
        "check_description": S3Checks.S3_PUBLIC_ACL_ACCESS
    },
    "S3_PUBLIC_POLICY_ACCESS": {
        "method_name": S3MethodAssociations.S3_PUBLIC_POLICY_ACCESS,
        "check_description": S3Checks.S3_PUBLIC_POLICY_ACCESS
    },
    "S3_CHECK_TAGS": {
        "method_name": S3MethodAssociations.S3_CHECK_TAGS,
        "check_description": S3Checks.S3_CHECK_TAGS
    }
}

best_practices_checks = {
    "S3_VERSIONING": {
        "method_name": S3MethodAssociations.S3_VERSIONING,
        "check_description": S3Checks.S3_VERSIONING
}
}

check_dict = {
    "SecurityCheck": security_checks,
    "BestPractices": best_practices_checks,
    "AccountSecurityCheck": {},
    "AccountBestPractices": {}
}



import json
import collections
import boto3
import botocore
import common.utils.helper as helper

from botocore.client import Config
from aws_services.kinesis_firehose import firehose_constants as constants
from common.constants import application_constants
from common.utils.initialize_logger import logger


class KinesisFirehose:
    """A class that checks the security settings of Amazon Kinesis Data Firehose delivery streams."""

    def __init__(self):
        """
            Initializes an Firehose client object with the specified maximum number of retries in specified region.
        """
        try:
            configuration = helper.get_configuration()
            self.firehose_client = boto3.client('firehose',config=configuration)
        except Exception as ex:
            logger.error("Error occurred while initializing Firehose client objects: %s", str(ex))
            raise ex

    def list_resources(self):
        """
        Returns a list of all Kinesis Data Firehose delivery streams associated with the Firehose client.

        Returns:
        - A list of Kinesis Data Firehose delivery stream names.

        Raises:
        - botocore.exceptions.ClientError: if there is an error communicating with AWS.
        """
        stream_list = []
        more_streams = True
        exclusive_start_stream_name = ''
        while more_streams:
            try:
                if exclusive_start_stream_name:
                    response = self.firehose_client.list_delivery_streams(ExclusiveStartDeliveryStreamName=exclusive_start_stream_name)
                else:
                    response = self.firehose_client.list_delivery_streams()
                stream_list.extend(response['DeliveryStreamNames'])
                more_streams = response['HasMoreDeliveryStreams']
                if more_streams:
                    exclusive_start_stream_name = response['DeliveryStreamNames'][-1]
            except botocore.exceptions.ClientError as ex:
                logger.error("Error occurred when listing Kinesis Data Firehose delivery streams: %s", str(ex))
                raise ex
        return stream_list

    def check_delivery_stream_encryption(self, delivery_stream_name):
        """
        Checks if server-side encryption with AWS KMS is enabled for source records in the Kinesis Data Firehose delivery stream.

        Returns:
            dict: A dictionary containing check result for the delivery stream.

        Raises:
            botocore.exceptions.ClientError: if there is an error communicating with AWS.
        """
        logger.info("Starting check for Kinesis Data Firehose delivery stream source encryption")

        try:
            # Describe the delivery stream to retrieve its configuration
            stream_description = self.firehose_client.describe_delivery_stream(DeliveryStreamName=delivery_stream_name)
            try:
                # Check if server-side encryption with AWS KMS is enabled for source records in the delivery stream
                if stream_description['DeliveryStreamDescription']['DeliveryStreamEncryptionConfiguration']['Status'] == 'ENABLED':
                    check_result = application_constants.ResultStatus.PASSED
                else:
                    check_result = application_constants.ResultStatus.FAILED
            except KeyError:
                check_result = application_constants.ResultStatus.FAILED

        except Exception as ex:
            logger.error("Error checking Kinesis Data Firehose delivery streams: %s: %s", delivery_stream_name, str(ex))
            check_result = application_constants.ResultStatus.UNKNOWN

        logger.info("Finished check for Kinesis Data Firehose delivery stream source encryption")
        return check_result



    def check_delivery_stream_tags(self, delivery_stream_name, required_tags=None):
        """
        Checks if the specified Firehose delivery stream has the required tags.

        Args:
            delivery_stream_name (str): The name of the Firehose delivery stream.

        Returns:
            dict: A dictionary containing the result of the check.
        """
        try:
            if required_tags is None:
                required_tags = application_constants.Generic.REQUIRED_TAGS

            if not required_tags:
                check_result = application_constants.ResultStatus.PASSED
            else:
                tags = self.firehose_client.list_tags_for_delivery_stream(DeliveryStreamName=delivery_stream_name)['Tags']
                missing_tags = [tag for tag in required_tags if tag.split(":")[0] not in [t["Key"] for t in tags]]
                check_result = application_constants.ResultStatus.PASSED if not any(missing_tags) else application_constants.ResultStatus.FAILED

            logger.info(f"Completed checking the tags for Firehose delivery stream {delivery_stream_name}")
            return check_result

        except botocore.exceptions.ClientError as ex:
            if ex.response['Error']['Code'] == 'ResourceNotFoundException':
                check_result = application_constants.ResultStatus.FAILED
            else:
                logger.error(f"An error occurred while checking the tags for Firehose delivery stream {delivery_stream_name}: {str(ex)}")
                check_result = application_constants.ResultStatus.UNKNOWN

            logger.info(f"Completed checking the tags for Firehose delivery stream {delivery_stream_name}")
            return check_result

        except Exception as ex:
            logger.exception(f"An error occurred while checking the tags for Firehose delivery stream {delivery_stream_name}: {str(ex)}")
            check_result = application_constants.ResultStatus.UNKNOWN

            logger.info(f"Completed checking the tags for Firehose delivery stream {delivery_stream_name}")
            return check_result



import os
import argparse
from datetime import datetime
from common.utils import helper
from common.service import dat_service
from common.constants import application_constants as constants
from common.utils.initialize_logger import logger

class Main:

    def generate_aws_audit_report(self):
        """
        Generates a audit report for AWS services by performing validations on the all AWS resources.

        Parameters:
        - None

        Returns:
        - None

        Raises:
        - Any exceptions thrown during the execution of the method.
        - ValueError: If AWS credentials are not configured.
        """
        try:
            # Check for AWS credentials
            helper.check_aws_credentials_exist()

            now = datetime.now()
            timestamp = now.strftime('%Y-%m-%d_%H-%M-%S')

            logger.info("Performing validation checks")

            config_data = dat_service.get_or_create_config()

            # get required tags to be validated and region name to be ran on
            constants.Generic.REQUIRED_TAGS = config_data["account_tags"]
            del config_data["account_tags"]

            service_info = dat_service.evaluate_standards(config_data)
            logger.info("Completed performing validation checks")

            helper.generate_audit_report(service_info, timestamp)

        except KeyboardInterrupt:
            logger.warning("User cancelled operation")
            raise
        except Exception as ex:
            logger.error("Error occurred when generating audit report: %s", str(ex))
            raise


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--region", help="Audit region", default="us-east-1")
    parser.add_argument("--path", help="Config.json's directory", default="")
    args = parser.parse_args()
    os.environ['AWS_DEFAULT_REGION'] = args.region
    os.environ['Config_Path']=args.path+"config.json"
    print(args.path)
    driver = Main()
    driver.generate_aws_audit_report()



import json
import boto3
import botocore

from aws_services.sqs import sqs_constants as constants
from common.constants import application_constants
from common.utils.initialize_logger import logger


class SQS:
    """A class that checks the security settings of Amazon SQS queues."""

    def __init__(self):
        self.sqs_client = boto3.client('sqs')

    def list_resources(self):
        """
        Returns a list of all SQS queues associated with the SQS client.

        Parameters:
        - None

        Returns:
        - A list of SQS queue URLS.

        Raises:
        - botocore.exceptions.ClientError: if there is an error communicating with AWS.
        """
        try:
            queueUrls = list()
            paginator = self.sqs_client.get_paginator('list_queues')
            pages = paginator.paginate(
                PaginationConfig={
                    'PageSize': 10
                }
            )
            for page in pages:
                if 'QueueUrls' in page:
                    queueUrls += page['QueueUrls']
            return queueUrls
        except botocore.exceptions.ClientError as ex:
            logger.error("Error occurred when listing SQS resources: %s", str(ex))
            raise ex

    def check_encryption_at_rest(self, queue_url):
        """
        Check if encryption at rest is enabled for a given SQS Queue.

        Args:
            queue_url (str): URL of the SQS Queue to check.

        Returns:
            None: This method updates the status of the check in the `self.service` dictionary.

        Raises:
            Exception: If any error occurs during the check.

        """
        logger.info("Checking for encryption at rest for queue %s", queue_url)

        try:
            response = self.sqs_client.get_queue_attributes(QueueUrl=queue_url, AttributeNames=['All'])
            if 'Attributes' in response and (
                    ('KmsMasterKeyId' in response['Attributes'] and len(response['Attributes']['KmsMasterKeyId'])) or (
                    'SqsManagedSseEnabled' in response['Attributes'] and response['Attributes'][
                'SqsManagedSseEnabled'] == 'true')):
                check_result = application_constants.ResultStatus.PASSED
            else:
                check_result = application_constants.ResultStatus.FAILED

        except self.sqs_client.exceptions.InvalidAttributeName as ex:
            logger.error("Invalid Attribute requested for queue %s: %s", queue_url, str(ex))
            check_result = application_constants.ResultStatus.UNKNOWN
        except self.sqs_client.exceptions.ClientError as ex:
            logger.error("Error checking encryption at rest for queue %s: %s", queue_url, str(ex))
            check_result = application_constants.ResultStatus.UNKNOWN
        except Exception as ex:
            logger.error("Error occurred during check for encryption at rest for queue %s: %s", queue_url, str(ex))
            raise ex

        logger.info("Completed checking for encryption at rest for queue %s", queue_url)
        return check_result

    def check_sqs_public_policy(self, queue_url):
        """
        Checks the queue policy of an SQS queue to determine if anonymous access is allowed.

        Args:
            queue_url (str): The url of the SQS queue to check.

        Returns:
            None: This method updates the status of the check in the `self.service` dictionary.
        """

        logger.info("Checking SQS Policy for anonymous access for queue %s", queue_url)
        try:
            response = self.sqs_client.get_queue_attributes(QueueUrl=queue_url, AttributeNames=['Policy'])
            if 'Attributes' in response and 'Policy' in response['Attributes'] and 'Statement' in \
                    response['Attributes']['Policy']:
                policy = json.loads(response['Attributes']['Policy'])

                for statement in policy['Statement']:
                    if 'Effect' in statement and statement['Effect'] == 'Allow' and 'Principal' in statement \
                            and (statement['Principal'] == '*' or ('AWS' in statement['Principal'] and statement['Principal']['AWS'] == "*")) \
                            and 'Condition' not in statement:
                        anonymous_access_enabled = True
                        break
                else:
                    anonymous_access_enabled = False
            else:
                anonymous_access_enabled = False

            check_result = application_constants.ResultStatus.FAILED if anonymous_access_enabled else application_constants.ResultStatus.PASSED

        except self.sqs_client.exceptions.ClientError as ex:
            logger.error("Error checking SQS policy for queue %s: %s", queue_url, str(ex))
            check_result = application_constants.ResultStatus.UNKNOWN

        logger.info("Completed checking SQS Policy for anonymous access for queue %s", queue_url)
        return check_result

    def check_queue_tags(self, queue_url, required_tags=None):
        """
        Checks if the specified SQS queue has the required tags.

        Args:
            queue_url (str): The url of the SQS queue.

        Returns:
            dict: A dictionary containing the result of the check.
        """
        try:
            if required_tags is None:
                required_tags = application_constants.Generic.REQUIRED_TAGS

            if not required_tags:
                check_result = application_constants.ResultStatus.PASSED
            else:
                response = self.sqs_client.list_queue_tags(QueueUrl=queue_url)
                if 'Tags' in response:
                    tags = response['Tags']
                    missing_tags = [tag for tag in required_tags if tag.split(":")[0] not in [t for t in tags]]
                    check_result = application_constants.ResultStatus.PASSED if not any(
                        missing_tags) else application_constants.ResultStatus.FAILED
                else:
                    check_result = application_constants.ResultStatus.FAILED

            logger.info(f"Completed checking the tags for SQS queue {queue_url}")
            return check_result

        except (botocore.exceptions.ClientError, Exception) as ex:
            logger.error(f"An error occurred while checking the tags for SQS queue {queue_url}: {str(ex)}")
            check_result = application_constants.ResultStatus.UNKNOWN

            logger.info(f"Completed checking the tags for SQS queue {queue_url}")
            return check_result

    def check_in_transit_encryption(self, queue_url):
        """
        Check if in-transit encryption is enabled for the specified SQS queue.

        Parameters:
            queue_url (str): The url of the sqs queue to check.

        Returns:
            None: This method updates the status of the check in the `self.service` dictionary.

        """
        logger.info("Checking for in-transit encryption using Secure Transport for queue %s", queue_url)
        try:
            response = self.sqs_client.get_queue_attributes(QueueUrl=queue_url, AttributeNames=['Policy'])
            if 'Attributes' in response and 'Policy' in response['Attributes'] and 'Statement' in \
                    response['Attributes']['Policy']:
                policy = json.loads(response['Attributes']['Policy'])
                statements = policy['Statement']

                # Check if any statement allows access over insecure transport
                for statement in statements:
                    if not (statement['Effect'] == 'Deny' and 'Condition' in statement and 'Bool' in statement[
                        'Condition'] and 'aws:SecureTransport' in statement['Condition']['Bool'] and \
                            statement['Condition']['Bool']['aws:SecureTransport'] == 'false') or \
                            (statement['Effect'] == 'Allow' and 'Condition' in statement and 'Bool' in statement[
                                'Condition'] and 'aws:SecureTransport' in statement['Condition']['Bool'] and \
                             statement['Condition']['Bool']['aws:SecureTransport'] == 'true'):
                        check_result = application_constants.ResultStatus.FAILED
                        break
                else:
                    check_result = application_constants.ResultStatus.PASSED
            else:
                check_result = application_constants.ResultStatus.FAILED


        except self.sqs_client.exceptions.ClientError as ex:
            logger.error("Error checking in-transit encryption for queue %s: %s", queue_url, str(ex))
            check_result = application_constants.ResultStatus.UNKNOWN

        logger.info("Completed checking for in-transit encryption using Secure Transport for queue %s", queue_url)
        return check_result

    def check_sqs_vpc_endpoint(self, queue_url):
        """
        Check if the specified SQS queue is accessible only via VPC Endpoint.

        Parameters:
            queue_url (str): The url of the sqs queue to check.

        Returns:
            None: This method updates the status of the check in the `self.service` dictionary.

        """
        logger.info("Checking for VPC endpoint of queue %s", queue_url)
        try:
            response = self.sqs_client.get_queue_attributes(QueueUrl=queue_url, AttributeNames=['Policy'])
            if 'Attributes' in response and 'Policy' in response['Attributes'] and 'Statement' in \
                    response['Attributes']['Policy']:
                policy = json.loads(response['Attributes']['Policy'])
                statements = policy['Statement']

                # Check if any statement allows access over insecure transport
                for statement in statements:
                    if not (statement['Effect'] == 'Deny' and 'Condition' in statement and 'StringNotEquals' in
                            statement['Condition'] and 'aws:SourceVpce' in statement['Condition']['StringNotEquals']) \
                            or (
                            statement['Effect'] == 'Allow' and 'Condition' in statement and 'StringEquals' in statement[
                        'Condition'] and 'aws:SourceVpce' in statement['Condition']['StringEquals']):
                        check_result = application_constants.ResultStatus.FAILED
                        break
                else:
                    check_result = application_constants.ResultStatus.PASSED
            else:
                check_result = application_constants.ResultStatus.FAILED

        except self.sqs_client.exceptions.ClientError as ex:
            logger.error("Error checking accessibility of SQS queue via VPC endpoint %s: %s", queue_url, str(ex))
            check_result = application_constants.ResultStatus.UNKNOWN

        logger.info("Completed checking for VPC endpoint of queue %s", queue_url)
        return check_result



class EMRChecks:
    EMR_CHECK_AT_REST_ENCRYPTION = "Amazon EMR clusters MUST be configured for at-rest encryption"
    EMR_IS_PRIVATE = "Amazon EMR Clusters MUST be deployed tp private subnets designated for data"
    EMR_LEAST_PRIVILEGE_SG = "Amazon EMR Clusters MUST have least privilege ingress and egress security group rules assigned"
    EMR_CHECK_TAGS = "MUST be tagged in accordance with tagging standards"

class EMRMethodAssociations:
    EMR_CHECK_AT_REST_ENCRYPTION = "check_emr_encryption_at_rest"
    EMR_LEAST_PRIVILEGE_SG = "check_least_privilege_sg"
    EMR_CHECK_TAGS = "check_emr_cluster_tags"
    EMR_IS_PRIVATE = "check_emr_cluster_in_private_subnet"

security_checks = {
    "EMR_CHECK_AT_REST_ENCRYPTION": {
        "method_name": EMRMethodAssociations.EMR_CHECK_AT_REST_ENCRYPTION,
        "check_description": EMRChecks.EMR_CHECK_AT_REST_ENCRYPTION
    },
    "EMR_LEAST_PRIVILEGE_SG": {
        "method_name": EMRMethodAssociations.EMR_LEAST_PRIVILEGE_SG,
        "check_description": EMRChecks.EMR_LEAST_PRIVILEGE_SG
    },
    "EMR_CHECK_TAGS": {
        "method_name": EMRMethodAssociations.EMR_CHECK_TAGS,
        "check_description": EMRChecks.EMR_CHECK_TAGS
    },
    "EMR_IS_PRIVATE": {
        "method_name": EMRMethodAssociations.EMR_IS_PRIVATE,
        "check_description": EMRChecks.EMR_IS_PRIVATE
    }
}

best_practices_checks = {
}

check_dict = {
    "SecurityCheck": security_checks,
    "BestPractices": best_practices_checks,
    "AccountSecurityCheck": {},
    "AccountBestPractices": {}
}



import boto3
import botocore
import common.utils.helper as helper

from common.constants import application_constants
from common.utils.initialize_logger import logger
from aws_services.dat_lambda import lambda_constants as constants

class DatLambda:
    """A class that checks the security settings of Amazon Lambda Service."""

    def __init__(self):
        """
            Initializes an lambda client object with the specified maximum number of retries in specified region.
        """
        try:
            configuration = helper.get_configuration()
            self.lambda_client = boto3.client('lambda',config=configuration)
        except Exception as ex:
            logger.error("Error occurred while initializing Lambda client objects: %s", str(ex))
            raise ex

    def list_resources(self):
        """
        Returns a list of all Lambda functions associated with the Lambda client.

        Returns:
        - A list of Lambda function names.

        Raises:
        - botocore.exceptions.ClientError: if there is an error communicating with AWS.
        """
        try:
            functions=[]
            paginator_functions = self.lambda_client.get_paginator('list_functions')
            for page in paginator_functions.paginate():
                functions += [function['FunctionName'] for function in page['Functions']]
            return functions
        except botocore.exceptions.ClientError as ex:
            logger.error("Error occurred when listing Lambda functions: %s", str(ex))
            raise ex


    def get_lambda_arn(self, lambda_function_name):
        """
        Generates the ARN for a given Lambda function.

        Args:
            lambda_function_name (str): The name of the Lambda function.

        Returns:
            str: The ARN of the Lambda function.
        """
        try:
            configuration = helper.get_configuration()
            client = boto3.client('sts',config=configuration)
            account_id = client.get_caller_identity().get('Account')
            region = client.meta.region_name
            return f"arn:aws:lambda:{region}:{account_id}:function:{lambda_function_name}"
        except Exception as ex:
            logger.error("Error occurred when calling get_lambda_arn: %s", str(ex))
            raise ex


    def check_lambda_function_tags(self, lambda_function_name, required_tags=None):
        """
        Checks if the specified Lambda function has the required tags.

        Args:
            lambda_function_name (str): The name of the Lambda function.
            required_tags (list): A list of required tags. Each tag should be in the format "Key:Value".

        Returns:
            dict: A dictionary containing the result of the check.
        """
        try:
            if required_tags is None:
                required_tags = application_constants.Generic.REQUIRED_TAGS

            if not required_tags:
                check_result = application_constants.ResultStatus.PASSED
            else:
                lambda_arn = self.get_lambda_arn(lambda_function_name)
                response = self.lambda_client.list_tags(Resource=lambda_arn)
                tags = response.get('Tags', {})
                missing_tags = [tag for tag in required_tags if tag.split(":")[0] not in tags]
                check_result = application_constants.ResultStatus.PASSED if not any(missing_tags) else application_constants.ResultStatus.FAILED

            logger.info("Completed checking the tags for Lambda function %s", lambda_function_name)
            return check_result

        except Exception as ex:
            logger.error("An error occurred while checking the tags for Lambda function %s: %s", lambda_function_name, str(ex))
            check_result = application_constants.ResultStatus.UNKNOWN
            return check_result



import collections
import boto3
import botocore

from common.constants import application_constants
from aws_services.rds import rds_constants as constants
from common.utils.initialize_logger import logger
from common.utils import helper

class RDS:


    def __init__(self):
        """
            Initializes an RDS,EC2 client object with the specified maximum number of retries in specified region.
        """
        try:
            configuration = helper.get_configuration()
            self.service = collections.defaultdict(dict)
            self.rds_client = boto3.client('rds',config=configuration)
            self.ec2_client = boto3.client('ec2',config=configuration)
            self.resource_list = []
        except Exception as ex:
            logger.error("Error occurred while initializing RDS and EC2 client objects: %s", str(ex))
            raise ex
       

    def list_resources(self):
        """
        Returns a list of all resources associated with the Boto client.
        """
        try:
            paginator_db_instances=self.rds_client.get_paginator('describe_db_instances')
            for page in paginator_db_instances.paginate():
                self.resource_list += [db_instance['DBInstanceIdentifier'] for db_instance in page['DBInstances'] ]
            return self.resource_list

        except botocore.exceptions.ClientError as ex:
            logger.error("Error occurred when listing RDS resources: %s", str(ex))
            raise


    def check_rds_public_accessibility(self, resource_name):

        logger.info("gathering the public accessibility settings for %s", resource_name)

        try:
            db_instances = self.rds_client.describe_db_instances()
            for db_instance in db_instances['DBInstances']:
                if resource_name == db_instance['DBInstanceIdentifier']:
                    if db_instance['PubliclyAccessible'] == True:
                        check_result = application_constants.ResultStatus.FAILED
                    else:
                        check_result = application_constants.ResultStatus.PASSED

        except self.rds_client.exceptions.ClientError as ex:
            error_code = ex.response['Error']['Code']
            if error_code == 'ServerSideEncryptionConfigurationNotFoundError':
                check_result = application_constants.ResultStatus.PASSED
            else:
                logger.error("error while gathering the public accessibility settings for %s: %s", resource_name, str(ex))
                check_result = application_constants.ResultStatus.UNKNOWN
        except Exception as ex:
            logger.error("gathering the public accessibility settings for %s: %s", resource_name, str(ex))
            raise ex

        logger.info("Completed fetching public accessibility settings for catalog ID %s", resource_name)
        return check_result

    def check_encryption_at_rest(self, resource_name):

        logger.info("gathering the encryption settings for %s", resource_name)
        try:
            db_instances = self.rds_client.describe_db_instances()
            for db_instance in db_instances['DBInstances']:
                if resource_name == db_instance['DBInstanceIdentifier']:
                    if db_instance['StorageEncrypted'] == True:
                        check_result = application_constants.ResultStatus.PASSED
                    else:
                        check_result = application_constants.ResultStatus.FAILED

        except self.rds_client.exceptions.ClientError as ex:
            error_code = ex.response['Error']['Code']
            if error_code == 'ServerSideEncryptionConfigurationNotFoundError':
                check_result = application_constants.ResultStatus.FAILED
            else:
                logger.error("error while gathering the encryption settings for %s: %s", resource_name, str(ex))
                check_result = application_constants.ResultStatus.UNKNOWN
        except Exception as ex:
            check_result = application_constants.ResultStatus.UNKNOWN
            logger.error("gathering the datacatalog settings for %s: %s", resource_name, str(ex))
            raise ex

        logger.info("Completed fetching encryption settings for catalog ID %s", resource_name)
        return check_result

    def check_rds_instance_cluster_snapshot(self, resource_name):

        logger.info("gathering the instance cluster snapshot settings for %s", resource_name)
        try:
            db_instances = self.rds_client.describe_db_instances()
            check_result = application_constants.ResultStatus.PASSED
            for db_instance in db_instances['DBInstances']:
                if resource_name == db_instance['DBInstanceIdentifier']:
                    response = self.rds_client.describe_db_snapshots(DBInstanceIdentifier=db_instance['DBInstanceIdentifier'])
                    for snapshot in response['DBSnapshots']:
                        if check_result == application_constants.ResultStatus.FAILED:
                            break
                        snapshot_attributes = self.rds_client.describe_db_snapshot_attributes(DBSnapshotIdentifier=snapshot['DBSnapshotIdentifier'])
                        for snapshot_attribute in snapshot_attributes['DBSnapshotAttributesResult']['DBSnapshotAttributes']:
                            if snapshot_attribute['AttributeName'] == 'restore' and snapshot_attribute['AttributeValues'] == ['all']:
                                check_result = application_constants.ResultStatus.FAILED
                                break
                    else:
                        check_result = application_constants.ResultStatus.PASSED
        except self.rds_client.exceptions.ClientError as ex:
            error_code = ex.response['Error']['Code']
            if error_code == 'ServerSideEncryptionConfigurationNotFoundError':
                check_result = application_constants.ResultStatus.FAILED
            else:
                logger.error("error while gathering the instance cluster snapshot settings for %s: %s", resource_name, str(ex))
                check_result = application_constants.ResultStatus.UNKNOWN
        except Exception as ex:
            logger.error("gathering the instance cluster snapshot settings for %s: %s", resource_name, str(ex))
            raise ex

        logger.info("Completed fetching instance cluster snapshot settings for catalog ID %s", resource_name)
        return check_result

    def check_rds_dedicated_vpc_sg(self, resource_name):

        logger.info("gathering the dedicated vpc settings for %s", resource_name)
        check_result = ''
        try:
            db_instances = self.rds_client.describe_db_instances()
            for db_instance in db_instances['DBInstances']:
                if resource_name == db_instance['DBInstanceIdentifier']:
                    if not db_instance['VpcSecurityGroups']:
                        check_result = application_constants.ResultStatus.FAILED
                    else:
                        check_result = application_constants.ResultStatus.PASSED

        except self.rds_client.exceptions.ClientError as ex:
            error_code = ex.response['Error']['Code']
            if error_code == 'ServerSideEncryptionConfigurationNotFoundError':
                check_result = application_constants.ResultStatus.FAILED
            else:
                logger.error("error while gathering the dedicated vpc settings for %s: %s", resource_name, str(ex))
                check_result = application_constants.ResultStatus.UNKNOWN
        except Exception as ex:
            check_result = application_constants.ResultStatus.UNKNOWN
            logger.error("gathering the dedicated vpc settings for %s: %s", resource_name, str(ex))
            raise ex

        logger.info("Completed fetching dedicated vpc settings for catalog ID %s", resource_name)
        return check_result

    def check_rds_private_subnet(self,resource_name):
        logger.info("gathering the private subnet data check for %s", resource_name)
        check_result = ''
        subnets = []
        private_subnet = []
        try:
            db_instances = self.rds_client.describe_db_instances()
            for db_instance in db_instances['DBInstances']:
                if resource_name == db_instance['DBInstanceIdentifier']:
                    for subnet_data in db_instance['DBSubnetGroup']['Subnets']:
                        subnets.append(subnet_data['SubnetIdentifier'])
                for subnet in subnets:
                    private_subnet.append(helper.check_subnet_has_igw(subnet_id=subnet))
                if any(private_subnet):
                    check_result = application_constants.ResultStatus.FAILED
                else:
                    check_result = application_constants.ResultStatus.PASSED

        except self.rds_client.exceptions.ClientError as ex:
            error_code = ex.response['Error']['Code']
            if error_code == 'ServerSideEncryptionConfigurationNotFoundError':
                check_result = application_constants.ResultStatus.FAILED
            else:
                logger.error("error while performing private subnet data check %s: %s", resource_name, str(ex))
                check_result = application_constants.ResultStatus.UNKNOWN
        except Exception as ex:
            logger.error("performing the private subnet data check for %s: %s", resource_name, str(ex))
            raise ex

        logger.info("Completed fetching private subnet data check %s", resource_name)
        return check_result

    def check_rds_ingress_egress(self, resource_name):

        logger.info("gathering the ingress engress settings for %s", resource_name)
        check_result = ''
        security_groups = []
        try:
            db_instances = self.rds_client.describe_db_instances()
            for db_instance in db_instances['DBInstances']:
                if resource_name == db_instance['DBInstanceIdentifier']:
                    for security_group in db_instance['VpcSecurityGroups']:
                        security_groups.append(security_group['VpcSecurityGroupId'])
                    security_group_details = self.ec2_client.describe_security_groups(GroupIds= security_groups)['SecurityGroups']
                    sg_rules = []
                    for sg in security_group_details:
                        sg_rules += sg['IpPermissions']
                    least_privilege = helper.is_least_privilege_sg(sg_rules)
                    if not least_privilege:
                        check_result = application_constants.ResultStatus.FAILED
                    else:
                        check_result = application_constants.ResultStatus.PASSED
        except self.rds_client.exceptions.ClientError as ex:
            error_code = ex.response['Error']['Code']
            if error_code == 'ServerSideEncryptionConfigurationNotFoundError':
                check_result = application_constants.ResultStatus.FAILED
            else:
                logger.error("error while gathering ingress engress  settings for %s: %s", resource_name, str(ex))
                check_result = application_constants.ResultStatus.UNKNOWN
        except Exception as ex:
            logger.error("gathering the ingress engress settings for %s: %s", resource_name, str(ex))
            raise ex

        logger.info("Completed fetching ingress engress settings for catalog ID %s", resource_name)
        return check_result


    def check_rds_tags(self, resource_name,required_tags=None):
        """
        Checks if the specified RDS Operation has the required tags.

        Returns:
            None: This method updates the status of the check in the `self.service` dictionary.
        """
        db_instances = self.rds_client.describe_db_instances()
        if required_tags is None:
            required_tags = application_constants.Generic.REQUIRED_TAGS
        for db_instance in db_instances['DBInstances']:
            if db_instance['DBInstanceIdentifier'] == resource_name:
                try:
                    if not required_tags:
                        check_result = application_constants.ResultStatus.PASSED
                    else:
                        logger.info("Checking the tags for %s", resource_name)
                        tags = self.rds_client.list_tags_for_resource(ResourceName=db_instance['DBInstanceArn'])
                        missing_tags = [tag for tag in required_tags if tag not in [t['Key'] for t in tags['TagList']]]
                        check_result = application_constants.ResultStatus.PASSED if not any(missing_tags) else application_constants.ResultStatus.FAILED
                    logger.info("Tags check for %s completed successfully.", resource_name)
                    return check_result
                except botocore.exceptions.ClientError as ex:
                    if ex.response['Error']['Code'] == 'NoSuchTagSet':
                        check_result = application_constants.ResultStatus.FAILED
                    else:
                        logger.error(f"An error occurred while checking the tags for {resource_name}: {str(ex)}")
                        check_result = application_constants.ResultStatus.UNKNOWN

                    logger.info(f"Completed checking the tags")
                    return check_result
                except Exception as ex:
                    logger.exception(f"An error occurred while checking the tags for {resource_name}: {str(ex)}")
                    check_result = application_constants.ResultStatus.UNKNOWN

                    logger.info(f"Completed checking the tags for resource : {resource_name}")
                    return check_result




import json
import boto3
import botocore
import common.utils.helper as helper
from aws_services.s3 import s3_constants as constants
from common.constants import application_constants
from common.utils.initialize_logger import logger

class S3:
    """A class that checks the security settings of Amazon S3 buckets."""

    def __init__(self):
        """
            Initializes an s3 client object with the specified maximum number of retries in specified region.
        """
        try:
            configuration = helper.get_configuration()
            self.s3_client = boto3.client('s3',config = configuration)

        except Exception as ex:
            logger.error("Error occurred while initializing s3 client objects: %s", str(ex))
            raise ex
        


    def list_resources(self):
        """
        Returns a list of all S3 buckets associated with the S3 client.

        Parameters:
        - None

        Returns:
        - A list of S3 bucket names.

        Raises:
        - botocore.exceptions.ClientError: if there is an error communicating with AWS.
        """
        try:
            buckets = [bucket['Name'] for bucket in self.s3_client.list_buckets()['Buckets']]
            return buckets
        except botocore.exceptions.ClientError as ex:
            logger.error("Error occurred when listing S3 resources: %s", str(ex))
            raise ex


    def check_encryption_at_rest(self, bucket_name):
        """
        Check if encryption at rest is enabled for a given S3 bucket.

        Args:
            bucket_name (str): Name of the S3 bucket to check.

        Returns:
            None: This method updates the status of the check in the `self.service` dictionary.

        Raises:
            Exception: If any error occurs during the check.

        """
        logger.info("Checking for encryption at rest for bucket %s", bucket_name)

        try:
            response = self.s3_client.get_bucket_encryption(Bucket=bucket_name)
            if 'ServerSideEncryptionConfiguration' in response and 'Rules' in response['ServerSideEncryptionConfiguration'] and len(response['ServerSideEncryptionConfiguration']['Rules']) > 0:
                check_result = application_constants.ResultStatus.PASSED
            else:
                check_result = application_constants.ResultStatus.FAILED

        except self.s3_client.exceptions.ClientError as ex:
            error_code = ex.response['Error']['Code']
            if error_code == 'ServerSideEncryptionConfigurationNotFoundError':
                check_result = application_constants.ResultStatus.FAILED
            else:
                logger.error("Error checking encryption at rest for bucket %s: %s", bucket_name, str(ex))
                check_result = application_constants.ResultStatus.UNKNOWN
        except Exception as ex:
            logger.error("Error occurred during check for encryption at rest for bucket %s: %s", bucket_name, str(ex))
            raise ex

        logger.info("Completed checking for encryption at rest for bucket %s", bucket_name)
        return check_result


    def check_in_transit_encryption(self, bucket_name):
        """
        Check if in-transit encryption is enabled for the specified S3 bucket.

        Parameters:
            bucket_name (str): The name of the S3 bucket to check.

        Returns:
            None: This method updates the status of the check in the `self.service` dictionary.

        """
        logger.info("Checking for in-transit encryption using Secure Transport for bucket %s", bucket_name)
        try:
            response = self.s3_client.get_bucket_policy(Bucket=bucket_name)
            policy = json.loads(response['Policy'])
            statements = policy['Statement']

            # Check if any statement allows access over insecure transport
            for statement in statements:
                if statement['Effect'] == 'Deny' and 'Condition' in statement and 'Bool' in statement['Condition'] and 'aws:SecureTransport' in statement['Condition']['Bool'] and statement['Condition']['Bool']['aws:SecureTransport'] == 'false':
                    check_result = application_constants.ResultStatus.PASSED
                    break
            else:
                check_result = application_constants.ResultStatus.FAILED

        except self.s3_client.exceptions.ClientError as ex:
            if ex.response['Error']['Code'] == 'NoSuchBucketPolicy':
                check_result = application_constants.ResultStatus.FAILED
            else:
                logger.error("Error checking in-transit encryption for bucket %s: %s", bucket_name, str(ex))
                check_result = application_constants.ResultStatus.UNKNOWN

        logger.info("Completed checking for in-transit encryption using Secure Transport for bucket %s", bucket_name)
        return check_result


    def check_s3_acl(self, bucket_name):
        """
        Checks the Access Control List (ACL) of an S3 bucket to determine if it is publicly accessible.

        Args:
            bucket_name (str): The name of the S3 bucket to check.

        Returns:
            None: This method updates the status of the check in the `self.service` dictionary.
        """

        logger.info("Checking S3 ACL for validating it is publicly accessible for bucket %s", bucket_name)
        try:
            response = self.s3_client.get_bucket_acl(Bucket=bucket_name)
            grants = response['Grants']

            for grant in grants:
                grantee = grant['Grantee']
                permission = grant['Permission']
                if 'URI' in grantee and grantee['URI'] in ('http://acs.amazonaws.com/groups/global/AllUsers', 'http://acs.amazonaws.com/groups/global/AuthenticatedUsers') and permission in ('FULL_CONTROL', 'WRITE'):
                    check_result = application_constants.ResultStatus.FAILED
                    break
            else:
                check_result = application_constants.ResultStatus.PASSED

        except self.s3_client.exceptions.NoSuchBucket:
            logger.error("Bucket %s not found", bucket_name)
            check_result = application_constants.ResultStatus.UNKNOWN

        except self.s3_client.exceptions.AccessDenied:
            logger.error("Access denied when checking ACL for bucket %s", bucket_name)
            check_result = application_constants.ResultStatus.UNKNOWN

        except self.s3_client.exceptions.ClientError as ex:
            logger.error("Error checking ACL for bucket %s: %s", bucket_name, str(ex))
            check_result = application_constants.ResultStatus.UNKNOWN

        else:
            logger.info("Successfully checked ACL for bucket %s", bucket_name)

        logger.info("Completed checking S3 ACL for validating it is publicly accessible for bucket %s", bucket_name)
        return check_result


    def check_s3_policy(self, bucket_name):
        """
        Checks the bucket policy of an S3 bucket to determine if public write access or non-secure access is allowed.

        Args:
            bucket_name (str): The name of the S3 bucket to check.

        Returns:
            None: This method updates the status of the check in the `self.service` dictionary.
        """

        logger.info("Checking S3 Policy for public write access or non-secure access for bucket %s", bucket_name)
        try:
            response = self.s3_client.get_bucket_policy(Bucket=bucket_name)
            policy = json.loads(response['Policy'])
            public_write_access_enabled = False

            for statement in policy['Statement']:
                if 'Effect' in statement and statement['Effect'] == 'Allow':
                    if 'Principal' in statement and statement['Principal'] == '*':
                        if 'Action' in statement and ('s3:PutObject' in statement['Action'] or 's3:PutObjectAcl' in statement['Action']):
                            public_write_access_enabled = True
                            break

            check_result = application_constants.ResultStatus.FAILED if public_write_access_enabled else application_constants.ResultStatus.PASSED


        except self.s3_client.exceptions.ClientError as ex:
            error_code = ex.response['Error']['Code']

            if error_code == 'NoSuchBucketPolicy':
                check_result = application_constants.ResultStatus.FAILED
            else:
                logger.error("Error checking S3 policy for bucket %s: %s", bucket_name, str(ex))
                check_result = application_constants.ResultStatus.UNKNOWN

        logger.info("Completed checking S3 Policy for public write access or non-secure access for bucket %s", bucket_name)
        return check_result


    def check_s3_versioning(self, bucket_name):
        """
        Checks the versioning status of an S3 bucket.

        Args:
            bucket_name (str): The name of the S3 bucket to check.

        Returns:
            None: This method updates the status of the check in the `self.service` dictionary.
        """

        logger.info("Checking if S3 versioning is enabled or not for bucket %s", bucket_name)
        try:
            response = self.s3_client.get_bucket_versioning(Bucket=bucket_name)
            check_result = application_constants.ResultStatus.PASSED if response.get('Status') == "Enabled" else application_constants.ResultStatus.FAILED
            logger.info("Successfully checked S3 versioning for bucket %s", bucket_name)

        except self.s3_client.exceptions.ClientError as ex:
            logger.error("Error checking S3 versioning for bucket %s: %s", bucket_name, str(ex))
            check_result = application_constants.ResultStatus.UNKNOWN

        logger.info("Completed if S3 versioning is enabled or not for bucket %s", bucket_name)
        return check_result


    def check_bucket_tags(self, bucket_name, required_tags=None):
        """
        Checks if the specified S3 bucket has the required tags.

        Args:
            bucket_name (str): The name of the S3 bucket.

        Returns:
            dict: A dictionary containing the result of the check.
        """
        try:
            if required_tags is None:
                required_tags = application_constants.Generic.REQUIRED_TAGS

            if not required_tags:
                check_result = application_constants.ResultStatus.PASSED
            else:
                tags = self.s3_client.get_bucket_tagging(Bucket=bucket_name)['TagSet']
                missing_tags = [tag for tag in required_tags if tag.split(":")[0] not in [t["Key"] for t in tags]]
                check_result = application_constants.ResultStatus.PASSED if not any(missing_tags) else application_constants.ResultStatus.FAILED

            logger.info(f"Completed checking the tags for S3 bucket {bucket_name}")
            return check_result

        except botocore.exceptions.ClientError as ex:
            if ex.response['Error']['Code'] == 'NoSuchTagSet':
                check_result = application_constants.ResultStatus.FAILED
            else:
                logger.error(f"An error occurred while checking the tags for S3 bucket {bucket_name}: {str(ex)}")
                check_result = application_constants.ResultStatus.UNKNOWN

            logger.info(f"Completed checking the tags for S3 bucket {bucket_name}")
            return check_result

        except Exception as ex:
            logger.exception(f"An error occurred while checking the tags for S3 bucket {bucket_name}: {str(ex)}")
            check_result = application_constants.ResultStatus.UNKNOWN

            logger.info(f"Completed checking the tags for S3 bucket {bucket_name}")
            return check_result



import json
import collections
import boto3
import botocore
import common.utils.helper as helper

from aws_services.kinesis_stream import kinesis_stream_constants as constants
from common.constants import application_constants
from common.utils.initialize_logger import logger

class KinesisStream:

    def __init__(self):
        """
            Initializes an Kinesis client object with the specified maximum number of retries in specified region.
        """
        try:
            configuration = helper.get_configuration()
            self.kinesis_stream_client = boto3.client('kinesis',config = configuration)
        except Exception as ex:
            logger.error("Error occurred while initializing kinesis client objects: %s", str(ex))
            raise ex


    def list_resources(self):
        """
        Returns a list of all Kinesis streams associated with the S3 client.

        Parameters:
        - None

        Returns:
        - A list of Kinesis streams.

        Raises:
        - botocore.exceptions.ClientError: if there is an error communicating with AWS.
        """
        try:
           kinesis_stream_names=[]
           paginator_streams=self.kinesis_stream_client.get_paginator('list_streams')
           for page in paginator_streams.paginate():
               kinesis_stream_names+=page['StreamNames']
           return kinesis_stream_names
        except botocore.exceptions.ClientError as ex:
            logger.error("Error occurred when listing Kinesis stream resources: %s", str(ex))
            raise ex


    def validate_kinesis_stream_tags(self, kinesis_stream_name,required_tags=None):
        """
        Check if the required tags are available for the specified Kinesis stream

        Parameters:
            kinesis_stream_name (str): The name of the Kinesis stream.

        Returns:
            None: This method updates the status of the check in the `self.service` dictionary.
        """
        logger.info("Checking Required Tags for Kinesis stream %s", kinesis_stream_name)
        try:
            if required_tags is None:
                required_tags = application_constants.Generic.REQUIRED_TAGS
            if not required_tags:
                check_result = application_constants.ResultStatus.PASSED

            kinesis_stream_tags= list()
            response = self.kinesis_stream_client.list_tags_for_stream(StreamName=kinesis_stream_name)
            has_more_tags: bool = response.get("HasMoreTags", None)
            kinesis_stream_tags += response['Tags']
            while has_more_tags:
                response = self.kinesis_stream_client.list_tags_for_stream(ExclusiveStartTagKey=response['Tags'][-1]['Key'])
                has_more_tags: bool = response.get("HasMoreTags", None)
                kinesis_stream_tags += response['Tags']
            valid_tags = []
            for tag in kinesis_stream_tags:
                key = tag.get('Key', None)
                if key in required_tags:
                    valid_tags.append(key)
            # Sorting the keys in tag for comparision
            required_tags.sort()
            valid_tags.sort()
            if required_tags == valid_tags:
                check_result = application_constants.ResultStatus.PASSED
            else:
                check_result = application_constants.ResultStatus.FAILED
        except self.kinesis_stream_client.exceptions.ClientError as ex:
            logger.error("Error Checking Required Tags for Kinesis stream named %s: %s", kinesis_stream_name, str(ex))
            check_result = application_constants.ResultStatus.UNKNOWN

        logger.info("Completed Checking Required Tags for Kinesis stream named %s", kinesis_stream_name)
        return check_result

    def check_in_transit_encryption(self, kinesis_stream_name):
        """
        Check if in-transit encryption is enabled for the specified Kinesis stream.

        Parameters:
            kinesis_stream_name (str): The name of the Kinesis stream.

        Returns:
            None: This method updates the status of the check in the `self.service` dictionary.

        """
        logger.info("Checking for in-transit encryption using Secure Transport for Kinesis stream %s", kinesis_stream_name)
        try:
            response = self.kinesis_stream_client.describe_stream(StreamName=kinesis_stream_name)
            stream_description = response.get('StreamDescription')
            if bool(stream_description) and 'EncryptionType' in stream_description and stream_description.get('EncryptionType'):
                encryption_type = stream_description.get('EncryptionType')
                if encryption_type == 'NONE':
                    check_result = application_constants.ResultStatus.FAILED
                elif encryption_type == 'KMS':
                    check_result = application_constants.ResultStatus.PASSED
                else:
                    logger.info("Since the encryption_type is neither KMS or NONE, we are setting the resultstatus to Unknown")
                    check_result = application_constants.ResultStatus.UNKNOWN

        except self.kinesis_stream_client.exceptions.ClientError as ex:
            logger.error("Error checking in-transit encryption for Kinesis stream named %s: %s", kinesis_stream_name, str(ex))
            check_result = application_constants.ResultStatus.UNKNOWN


        logger.info("Completed checking for in-transit encryption using Secure Transport for Kinesis stream named %s", kinesis_stream_name)
        return check_result



class KinesisStreamChecks:
    KINESIS_STREAM_CHECK_IN_TRANSIT_ENCRYPTION = "Kinesis Data Streams MUST be configured for server side encryption"
    KINESIS_STREAM_CHECK_TAGS = "MUST be tagged in accordance with tagging standards"

class KinesisStreamMethodAssociations:
    KINESIS_STREAM_CHECK_IN_TRANSIT_ENCRYPTION = "check_in_transit_encryption"
    KINESIS_STREAM_CHECK_TAGS = "validate_kinesis_stream_tags"

security_checks = {
    "KINESIS_STREAM_CHECK_IN_TRANSIT_ENCRYPTION": {
        "method_name": KinesisStreamMethodAssociations.KINESIS_STREAM_CHECK_IN_TRANSIT_ENCRYPTION,
        "check_description": KinesisStreamChecks.KINESIS_STREAM_CHECK_IN_TRANSIT_ENCRYPTION
    },
    "KINESIS_STREAM_CHECK_TAGS": {
        "method_name": KinesisStreamMethodAssociations.KINESIS_STREAM_CHECK_TAGS,
        "check_description": KinesisStreamChecks.KINESIS_STREAM_CHECK_TAGS
    }
}

best_practices_checks = { }

kinesis_stream_dict = {
    "SecurityCheck": security_checks,
    "BestPractices": best_practices_checks,
    "AccountSecurityCheck": {},
    "AccountBestPractices": {}
}



import os
from aws_services.dynamodb.dynamodb_constants import dynamodb_dict
from aws_services.s3.s3_constants import check_dict as s3_check_dict
from aws_services.kinesis_firehose.firehose_constants import check_dict as firehose_check_dict
from aws_services.glue.glue_constants import glue_dict
from aws_services.rds.rds_constants import rds_dict
from aws_services.kinesis_stream.kinesis_stream_constants import kinesis_stream_dict
from aws_services.emr.emr_constants import check_dict as emr_dict
from aws_services.api_gateway.api_gateway_constants import api_gateway_dict
from aws_services.dat_lambda.lambda_constants import check_dict as lambda_dict
from aws_services.fargate.fargate_constants import fargate_dict as fargate_dict
from aws_services.sqs.sqs_constants import check_dict as sqs_check_dict
from aws_services.redshift.redshift_constants import redshift_dict

class AWSServices:
    S3 = "s3"
    KINESIS_STREAMS = "Kinesis Stream"
    KINESIS_FIREHOSE = "Kinesis Firehose"
    GLUE = "Glue"
    RDS = "RDS"
    EMR = "EMR"
    API_GATEWAY = "Api Gateway"
    LAMBDA = "Lambda"
    FARGATE = "Fargate"
    SQS = "SQS"
    REDSHIFT = "Redshift"
    DYNAMODB = "Dynamodb"

class ResultStatus:
    PASSED = "Passed"
    FAILED = "Failed"
    UNKNOWN = "Unknown"
    DISABLED = "Disabled"

class Type:
    SECURITY_CHECK = "SecurityCheck"
    BEST_PRACTICES = "BestPractices"
    ACC_SECURITY_CHECK = "AccountSecurityCheck"
    ACC_BEST_PRACTICES = "AccountBestPractices"

overall_check_dict = {
    AWSServices.S3: s3_check_dict,
    AWSServices.GLUE: glue_dict,
    AWSServices.RDS : rds_dict,
    AWSServices.KINESIS_FIREHOSE: firehose_check_dict,
    AWSServices.GLUE: glue_dict,
    AWSServices.KINESIS_STREAMS: kinesis_stream_dict,
    AWSServices.EMR: emr_dict,
    AWSServices.API_GATEWAY: api_gateway_dict,
    AWSServices.LAMBDA: lambda_dict,
    AWSServices.FARGATE: fargate_dict,
    AWSServices.SQS: sqs_check_dict,
    AWSServices.REDSHIFT: redshift_dict,
    AWSServices.DYNAMODB: dynamodb_dict
}

class Generic:
    MAX_RETRIES = 3
    REGION_NAME = ""
    REQUIRED_TAGS = []



from aws_services.dynamodb.dynamodb_service import DynamoDB
from aws_services.redshift.redshift_service import Redshift
from aws_services.s3.s3_service import S3
from aws_services.kinesis_firehose.kinesis_firehose_service import KinesisFirehose
from aws_services.glue.glue_service import Glue
from aws_services.rds.rds_service import RDS
from aws_services.kinesis_stream.kinesis_stream_service import KinesisStream
from aws_services.emr.emr_service import EMR
from aws_services.api_gateway.api_gateway_service import APIGateway
from aws_services.dat_lambda.lambda_service import DatLambda
from aws_services.fargate.fargate_service import Fargate
from aws_services.sqs.sqs_service import SQS
from common.constants import application_constants as constants


def initialize_members():
    instantiate_members = {
        constants.AWSServices.S3: S3(),
        constants.AWSServices.GLUE: Glue(),
        constants.AWSServices.RDS: RDS(),
        constants.AWSServices.KINESIS_FIREHOSE: KinesisFirehose(),
        constants.AWSServices.KINESIS_STREAMS: KinesisStream(),
        constants.AWSServices.EMR: EMR(),
        constants.AWSServices.API_GATEWAY: APIGateway(),
        constants.AWSServices.LAMBDA: DatLambda(),
        constants.AWSServices.FARGATE: Fargate(),
        constants.AWSServices.SQS: SQS(),
        constants.AWSServices.REDSHIFT: Redshift(),
        constants.AWSServices.DYNAMODB: DynamoDB(),
    }
    return instantiate_members



class APIGatewayChecks:
    AUTHORISATION_AUTHENTICATION = "authorisation_authentication"
    ENDPOINT_PUBLIC_ACCESSIBILITY = "endpoint_public_accessibility"
    TAGS = "Required Tags"

class ApiGatewayMethodAssociations:
    API_GATEWAY_AUTHORISATION_AUTHENTICATION = "check_authorisation_authentication"
    API_GATEWAY_ENDPOINT_PUBLIC_ACCESSIBILITY = "check_endpoint_public_accessibility"
    API_GATEWAY_TAGS = "check_api_gateway_tags"

security_checks = {
    "API_GATEWAY_AUTHORISATION_AUTHENTICATION" : {
        "method_name": ApiGatewayMethodAssociations.API_GATEWAY_AUTHORISATION_AUTHENTICATION,
        "check_description": APIGatewayChecks.AUTHORISATION_AUTHENTICATION
    },
    "API_GATEWAY_TAGS": {
        "method_name": ApiGatewayMethodAssociations.API_GATEWAY_TAGS,
        "check_description": APIGatewayChecks.TAGS
    }
}

best_practices_checks = {
    "API_GATEWAY_ENDPOINT_PUBLIC_ACCESSIBILITY": {
        "method_name": ApiGatewayMethodAssociations.API_GATEWAY_ENDPOINT_PUBLIC_ACCESSIBILITY,
        "check_description": APIGatewayChecks.ENDPOINT_PUBLIC_ACCESSIBILITY
    }
}
api_gateway_dict = {
    "SecurityCheck": security_checks,
    "BestPractices": best_practices_checks,
    "AccountSecurityCheck": {},
    "AccountBestPractices": {}
}


class KinesisFirehose:
    FIREHOSE_CHECK_SSE_ENCRYPTION = "Kinesis Data Firehose MUST be configured for at-rest encryption using SSE"
    FIREHOSE_CHECK_TAGS = "MUST be tagged in accordance with tagging standards"

class FirehoseMethodAssociations:
    FIREHOSE_CHECK_SSE_ENCRYPTION = "check_delivery_stream_encryption"
    FIREHOSE_CHECK_TAGS = "check_delivery_stream_tags"

security_checks = {
    "FIREHOSE_CHECK_SSE_ENCRYPTION": {
        "method_name": FirehoseMethodAssociations.FIREHOSE_CHECK_SSE_ENCRYPTION,
        "check_description": KinesisFirehose.FIREHOSE_CHECK_SSE_ENCRYPTION
    },
    "FIREHOSE_CHECK_TAGS": {
        "method_name": FirehoseMethodAssociations.FIREHOSE_CHECK_TAGS,
        "check_description": KinesisFirehose.FIREHOSE_CHECK_TAGS
    }
}

best_practices_checks = {
}

check_dict = {
    "SecurityCheck": security_checks,
    "BestPractices": best_practices_checks,
    "AccountSecurityCheck": {},
    "AccountBestPractices": {}
}



class FargateChecks:
    FARGATE_PUBLIC_ACESS = "Amazon ECS Fargate Tasks should not have Public Ip"
    FARGATE_PRIVATE_SUBNET = "Amazon ECS Fargate Taks should be in Private Subnet"
    FARGATE_SG_RULES = "Amazon ECS Fargate Tasks should not allow traffic from anywhere"
    FARGATE_TAGS = "Amazon ECS Fargate Tasks MUST be tagged in accordance with tagging standards"


class FargateMethodAssociations:
    FARGATE_PUBLIC_ACESS = "check_public_access"
    FARGATE_PRIVATE_SUBNET = "check_if_in_private_subnet"
    FARGATE_SG_RULES = "check_ip_permissions"
    FARGATE_TAGS = "check_fargate_tags"


security_checks = {
    "CHECK_FARGATE_PUBLIC_ACESS": {
    "method_name":FargateMethodAssociations.FARGATE_PUBLIC_ACESS,
    "check_description":FargateChecks.FARGATE_PUBLIC_ACESS
    },
    "CHECK_FARGATE_PRIVATE_SUBNET": {
    "method_name":FargateMethodAssociations.FARGATE_PRIVATE_SUBNET,
    "check_description":FargateChecks.FARGATE_PRIVATE_SUBNET
    },
    "CHECK_FARGATE_SG_RULES": {
    "method_name":FargateMethodAssociations.FARGATE_SG_RULES,
    "check_description":FargateChecks.FARGATE_SG_RULES
    },
    "CHECK_FARGATE_TAGS":{ 
    "method_name":FargateMethodAssociations.FARGATE_TAGS,
    "check_description":FargateChecks.FARGATE_TAGS
    }

}

best_practices_checks = {
}

fargate_dict = {
    "SecurityCheck": security_checks,
    "BestPractices": best_practices_checks,
    "AccountSecurityCheck": {},
    "AccountBestPractices": {}
}



from distutils.command.register import register as register_orig
from distutils.command.upload import upload as upload_orig
from setuptools import find_packages
import setuptools
import os

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="toolnew4",                
    version="1.0",                        
    author="Bhanuja",  
    author_email="abhanuja@presidio.com",
    license= 'MIT',              
    description="Data And Analytics Audit Tool",
    long_description=long_description,     
    long_description_content_type="text/markdown",
    packages=find_packages('src'),
    package_dir={'': 'src'},
    py_modules=['main'],
    entry_points={
        'console_scripts': [
            'my_command=src:main'
        ]
    },
    include_package_data=True,
    python_requires='>=3.6',                
    install_requires=["boto3==1.26.85","nose==1.3.7","Jinja2==3.1.2","moto==4.1.4","docker==6.0.1","pyparsing==3.0.9","openapi_spec_validator==0.5.6"]                    
)


import collections
import itertools

import boto3
import botocore

from common.constants import application_constants
from common.utils import helper
from common.utils.initialize_logger import logger


class Redshift:
    """A class that checks the security settings of Amazon Redshift Cluster."""

    def __init__(self):
        """
            Initializes a Redshift,EC2 client object with the specified maximum number of retries in specified region.
        """
        try:
            configuration = helper.get_configuration()
            self.service = collections.defaultdict(dict)
            self.redshift_client = boto3.client('redshift', config=configuration)
            self.ec2_client = boto3.client('ec2', config=configuration)
            self.resource_list = []
        except Exception as ex:
            logger.error("Error occurred while initializing Redshift and EC2 client objects: %s", str(ex))
            raise ex

    def list_resources(self):
        """
        Returns a list of all redshift clusters associated with the boto client.

        Parameters:
        - None

        Returns:
        - A list of Redshift cluster Identifiers.

        Raises:
        - botocore.exceptions.ClientError: if there is an error communicating with AWS.
        """
        try:
            paginator_db_instances = self.redshift_client.get_paginator('describe_clusters')
            for page in paginator_db_instances.paginate(PaginationConfig={'PageSize': 20}):
                self.resource_list += [cluster['ClusterIdentifier'] for cluster in page['Clusters']]
            return self.resource_list

        except botocore.exceptions.ClientError as ex:
            logger.error("Error occurred when listing RDS resources: %s", str(ex))
            raise

    def check_encryption_at_rest(self, cluster_identifier):
        """
        Check if encryption at rest is enabled for a given redshift .

        Args:
            cluster_identifier (str): Cluster Identifier of the Redshift Cluster to check.

        Returns:
            None: This method updates the status of the check in the `self.service` dictionary.

        Raises:
            ClientError: if there is an error communicating with AWS.
            ClusterNotFoundFault: if the requested cluster not found
            Exception: If any error occurs during the check.

        """
        logger.info("gathering the encryption settings for %s", cluster_identifier)
        try:
            cluster = self.redshift_client.describe_clusters(ClusterIdentifier=cluster_identifier)['Clusters'][0]
            if cluster['Encrypted']:
                check_result = application_constants.ResultStatus.PASSED
            else:
                check_result = application_constants.ResultStatus.FAILED
        except (
                self.redshift_client.exceptions.ClientError,
                self.redshift_client.exceptions.ClusterNotFoundFault) as ex:
            logger.error("error while gathering the encryption settings for %s: %s", cluster_identifier, str(ex))
            check_result = application_constants.ResultStatus.UNKNOWN
        except Exception as ex:
            logger.error("error while gathering the encryption settings for %s: %s", cluster_identifier, str(ex))
            raise ex

        logger.info("Completed fetching encryption settings for redshift cluster : %s", cluster_identifier)
        return check_result

    def check_redshift_public_accessibility(self, cluster_identifier):
        """
        Check if given redshift cluster allows anonymous access.

        Args:
            cluster_identifier (str): Cluster Identifier of the Redshift Cluster to check.

        Returns:
            None: This method updates the status of the check in the `self.service` dictionary.

        Raises:
            ClientError: if there is an error communicating with AWS.
            ClusterNotFoundFault: if the requested cluster not found
            Exception: If any error occurs during the check.

        """
        logger.info("gathering the public accessibility settings for %s", cluster_identifier)
        try:
            cluster = self.redshift_client.describe_clusters(ClusterIdentifier=cluster_identifier)['Clusters'][0]
            if cluster['PubliclyAccessible']:
                check_result = application_constants.ResultStatus.FAILED
            else:
                check_result = application_constants.ResultStatus.PASSED
        except (self.redshift_client.exceptions.ClientError,
                self.redshift_client.exceptions.ClusterNotFoundFault) as ex:
            logger.error("error while gathering the public accessibility settings for %s: %s", cluster_identifier,
                         str(ex))
            check_result = application_constants.ResultStatus.UNKNOWN
        except Exception as ex:
            logger.error("error while gathering the public accessibility settings for %s: %s", cluster_identifier,
                         str(ex))
            raise ex
        logger.info("Completed fetching public accessibility settings for redshift cluster : %s", cluster_identifier)
        return check_result

    def check_redshift_private_subnet(self, cluster_identifier):
        """
       Check if given redshift cluster is only in a private subnet.

       Args:
           cluster_identifier (str): Cluster Identifier of the Redshift Cluster to check.

       Returns:
           None: This method updates the status of the check in the `self.service` dictionary.

       Raises:
           ClientError: if there is an error communicating with AWS.
           ClusterNotFoundFault: if the requested cluster not found
           ClusterSubnetGroupNotFoundFault: if subnet group of the given cluster not found
           Exception: If any error occurs during the check.

       """
        logger.info("gathering the private subnet data check for %s", cluster_identifier)
        private_subnet = []
        try:
            subnet_group_name = \
                self.redshift_client.describe_clusters(ClusterIdentifier=cluster_identifier)['Clusters'][0][
                    'ClusterSubnetGroupName']
            subnet_groups = \
                self.redshift_client.describe_cluster_subnet_groups(ClusterSubnetGroupName=subnet_group_name)[
                    'ClusterSubnetGroups']
            subnets = list(itertools.chain(*[subnet_group['Subnets'] for subnet_group in subnet_groups]))
            for subnet in subnets:
                private_subnet.append(helper.check_subnet_has_igw(subnet_id=subnet['SubnetIdentifier']))
            if any(private_subnet):
                check_result = application_constants.ResultStatus.FAILED
            else:
                check_result = application_constants.ResultStatus.PASSED
        except (self.redshift_client.exceptions.ClientError, self.redshift_client.exceptions.ClusterNotFoundFault,
                self.redshift_client.exceptions.ClusterSubnetGroupNotFoundFault) as ex:
            logger.error("error while performing private subnet data check for %s: %s", cluster_identifier, str(ex))

        except Exception as ex:
            logger.error("performing the private subnet data check for %s: %s", cluster_identifier, str(ex))
            raise ex

        logger.info("Completed fetching private subnet data check %s", cluster_identifier)
        return check_result

    def check_redshift_dedicated_security_group(self, cluster_identifier):
        """
        Check if given redshift cluster has a dedicated security group.

        Args:
            cluster_identifier (str): Cluster Identifier of the Redshift Cluster to check.

        Returns:
            None: This method updates the status of the check in the `self.service` dictionary.

        Raises:
            ClientError: if there is an error communicating with AWS.
            ClusterNotFoundFault: if the requested cluster not found
            Exception: If any error occurs during the check.

        """
        logger.info("gathering the dedicated vpc security group settings for %s", cluster_identifier)
        try:
            security_groups = \
                self.redshift_client.describe_clusters(ClusterIdentifier=cluster_identifier)['Clusters'][0][
                    'VpcSecurityGroups']
            if len(security_groups):
                security_group_ids = [security_group['VpcSecurityGroupId'] for security_group in security_groups]
                paginator = self.ec2_client.get_paginator('describe_network_interfaces')
                pages = paginator.paginate(Filters=[{
                    'Name': 'group-id',
                    'Values': security_group_ids
                }], PaginationConfig={'PageSize': 10})
                network_interfaces = [page['NetworkInterfaces'] for page in pages]
                network_interfaces = list(itertools.chain(*network_interfaces))
                if len(network_interfaces) == 1:
                    check_result = application_constants.ResultStatus.PASSED
                else:
                    check_result = application_constants.ResultStatus.FAILED
            else:
                check_result = application_constants.ResultStatus.FAILED

        except (self.redshift_client.exceptions.ClientError, self.redshift_client.exceptions.ClusterNotFoundFault) as ex:
            logger.error("error while gathering the dedicated vpc settings for %s: %s", cluster_identifier, str(ex))
            check_result = application_constants.ResultStatus.UNKNOWN
        except Exception as ex:
            check_result = application_constants.ResultStatus.UNKNOWN
            logger.error("gathering the dedicated vpc settings for %s: %s", cluster_identifier, str(ex))
            raise ex

        logger.info("Completed fetching dedicated vpc settings for redshift cluster %s", cluster_identifier)
        return check_result

    def check_redshift_ingress_egress(self, cluster_identifier):
        """
         Check if given redshift cluster has security group only with least ingress and egress rules.

         Args:
             cluster_identifier (str): Cluster Identifier of the Redshift Cluster to check.

         Returns:
             None: This method updates the status of the check in the `self.service` dictionary.

         Raises:
             ClientError: if there is an error communicating with AWS.
             ClusterNotFoundFault: if the requested cluster not found
             Exception: If any error occurs during the check.

         """
        logger.info("gathering the ingress engress settings for %s", cluster_identifier)
        try:
            cluster = self.redshift_client.describe_clusters(ClusterIdentifier=cluster_identifier)['Clusters'][0]
            security_groups = [security_group['VpcSecurityGroupId'] for security_group in cluster['VpcSecurityGroups']]
            security_group_details = self.ec2_client.describe_security_groups(GroupIds=security_groups)[
                'SecurityGroups']
            sg_rules = []
            for security_group in security_group_details:
                sg_rules += security_group['IpPermissions']
            least_privilege = helper.is_least_privilege_sg(sg_rules)

            check_result = application_constants.ResultStatus.PASSED if least_privilege else application_constants.ResultStatus.FAILED

        except (
                self.redshift_client.exceptions.ClientError,
                self.redshift_client.exceptions.ClusterNotFoundFault) as ex:
            logger.error("error while gathering the ingress egress settings for %s: %s", cluster_identifier, str(ex))
            check_result = application_constants.ResultStatus.UNKNOWN
        except Exception as ex:
            logger.error("error while gathering the ingress egress settings for %s: %s", cluster_identifier, str(ex))
            raise ex

        logger.info("Completed fetching ingress egress settings for redshift cluster %s", cluster_identifier)
        return check_result

    def check_redshift_tags(self, cluster_identifier, required_tags=None):
        """
        Checks if the specified Redshift cluster has the required tags.

        Args:
             cluster_identifier (str): Cluster Identifier of the Redshift Cluster to check.

        Returns:
            None: This method updates the status of the check in the `self.service` dictionary.
        """

        if required_tags is None:
            required_tags = application_constants.Generic.REQUIRED_TAGS

        if not required_tags:
            check_result = application_constants.ResultStatus.PASSED
        else:
            logger.info("Checking the tags for %s", cluster_identifier)
            try:
                tags = self.redshift_client.describe_clusters(ClusterIdentifier=cluster_identifier)['Clusters'][0][
                    'Tags']
                missing_tags = [tag for tag in required_tags if tag not in [t['Key'] for t in tags]]
                check_result = application_constants.ResultStatus.PASSED if not any(
                    missing_tags) else application_constants.ResultStatus.FAILED
            except Exception as ex:
                logger.exception(f"An error occurred while checking the tags for {cluster_identifier}: {str(ex)}")
                check_result = application_constants.ResultStatus.UNKNOWN

        logger.info(f"Completed checking the tags for resource : {cluster_identifier}")
        return check_result



import collections
import boto3
import botocore
import common.utils.helper as helper
from common.constants import application_constants
from aws_services.api_gateway import api_gateway_constants as constants
from common.utils.initialize_logger import logger

class APIGateway:
    """A class that checks the security settings of API GATEWAY"""

    def __init__(self):
        """
            Initializes an apigateway and iam client object with the specified maximum number of retries in specified region.
        """
        try:
            configuration = helper.get_configuration()
            self.api_gateway_client = boto3.client('apigatewayv2',config=configuration)
            self.api_client = boto3.client('apigateway',config=configuration)
            self.iam_client = boto3.client('iam',config=configuration)

        except Exception as ex:
            logger.error("Error occurred while initializing apigateway and iam client objects: %s", str(ex))
            raise ex
        

    def init_connections(self):
        self.apis={"Items":[]}
        self.rest_apis={"items":[]}
        paginator_get_apis=self.api_gateway_client.get_paginator('get_apis')
        for page in paginator_get_apis.paginate():
            self.apis['Items']+=page['Items']
        paginator_get_rest_apis=self.api_client.get_paginator('get_rest_apis')
        for page in paginator_get_rest_apis.paginate():
            self.rest_apis['items']+=page['items']
        self.resource_list = []

    def list_resources(self):
        """
        Returns a list of all resources associated with the Glue client.
        """
        
        try:
            self.init_connections()
            for apiid in self.apis['Items']:
                self.resource_list.append(apiid['ApiId'])
            for rest_api in self.rest_apis['items']:
                self.resource_list.append(rest_api['id'])
            return self.resource_list   
        except botocore.exceptions.ClientError as ex:
            logger.error("Error occurred when listing api gateway resources: %s", str(ex))
            raise
    
    def check_endpoint_public_accessibility(self,resource_name):
        try:
            self.init_connections()
            rest_api_id=[]
            if len(self.rest_apis['items'])==0:
                check_result=application_constants.ResultStatus.UNKNOWN
            for rest_api in self.rest_apis['items']:
                rest_api_id.append(rest_api['id'])
            if resource_name not in rest_api_id:
                check_result= application_constants.ResultStatus.UNKNOWN
            else:
                response = self.api_client.get_rest_api(restApiId = resource_name)
                if response['endpointConfiguration']['types'] != ['PRIVATE']:
                    check_result = application_constants.ResultStatus.FAILED
                elif response['endpointConfiguration']['types'] != ['REGIONAL'] or response['endpointConfiguration']['types'] != ['EDGE']:
                    check_result = application_constants.ResultStatus.PASSED
            
        except self.api_gateway_client.exceptions.ClientError as ex:
                error_code = ex.response['Error']['Code']
                if error_code == 'ServerSideEncryptionConfigurationNotFoundError':
                    check_result = application_constants.ResultStatus.DISABLED
                else:
                    logger.error("error while gathering the apigateway settings for %s: %s", resource_name, str(ex))
                    check_result = application_constants.ResultStatus.UNKNOWN
        except Exception as ex:
            logger.error("gathering the apigateway settings for %s: %s", resource_name, str(ex))
            raise ex

        logger.info("Completed fetching apigateway settings for catalog ID %s", resource_name)
        return check_result
    
    def check_authorisation_authentication(self,resource_name):
        try:
            self.init_connections()
            for rest_api in self.rest_apis['items']:
                if resource_name == rest_api['id']:
                    response = self.api_client.get_authorizers(restApiId = resource_name)
                    if response['items']:
                        check_result = application_constants.ResultStatus.PASSED
                    else:
                        check_result = application_constants.ResultStatus.FAILED
            for api in self.apis['Items']:
                if resource_name == api['ApiId']:
                    response = self.api_gateway_client.get_authorizers(ApiId = resource_name)
                    if response['Items']:
                        check_result = application_constants.ResultStatus.PASSED
                    else:
                        check_result = application_constants.ResultStatus.FAILED
        except self.api_gateway_client.exceptions.ClientError as ex:
                error_code = ex.response['Error']['Code']
                if error_code == 'ServerSideEncryptionConfigurationNotFoundError':
                    check_result = application_constants.ResultStatus.DISABLED
                else:
                    logger.error("error while gathering the apigateway settings for %s: %s", resource_name, str(ex))
                    check_result = application_constants.ResultStatus.UNKNOWN
        except Exception as ex:
            logger.error("gathering the apigateway settings for %s: %s", resource_name, str(ex))
            raise ex

        logger.info("Completed fetching apigateway settings for catalog ID %s", resource_name)
        return check_result

    def check_api_gateway_tags(self,resource_name,required_tags=None):
        try:
            self.init_connections()
            missing_tags= []
            check_result = None
            if required_tags is None:
                required_tags = application_constants.Generic.REQUIRED_TAGS
            if not required_tags:
                check_result = application_constants.ResultStatus.PASSED
            else:
                for rest_api in self.rest_apis['items']:
                    if resource_name == rest_api['id']:
                        if 'tags' in rest_api:
                                tags = rest_api['tags']
                                missing_tags = [tag for tag in required_tags if tag not in [t for t in tags]]
                                check_result = application_constants.ResultStatus.PASSED if not any(missing_tags) else application_constants.ResultStatus.FAILED
                        else:
                            check_result = application_constants.ResultStatus.FAILED
                for api in self.apis['Items']:
                    if resource_name == api['ApiId']:
                        if 'Tags' in api:
                            tags = api['Tags']
                            missing_tags = [tag for tag in required_tags if tag not in [t for t in tags]]
                            check_result = application_constants.ResultStatus.PASSED if not any(missing_tags) else application_constants.ResultStatus.FAILED
                        else:
                            check_result = application_constants.ResultStatus.FAILED
        except self.api_gateway_client.exceptions.ClientError as ex:
                error_code = ex.response['Error']['Code']
                if error_code == 'ServerSideEncryptionConfigurationNotFoundError':
                    check_result = application_constants.ResultStatus.DISABLED
                else:
                    logger.error("error while gathering the apigateway settings for %s: %s", resource_name, str(ex))
                    check_result = application_constants.ResultStatus.UNKNOWN
        except Exception as ex:
            logger.error("gathering the apigateway settings for %s: %s", resource_name, str(ex))
            raise ex
        
        return check_result

        



class GlueChecks:
    GLUE_CHECK_DATA_CATALOG_ENCRYPTION = "AWS Glue data catalog objects MUST be configured for at-rest encryption"
    GLUE_CHECK_CONNECTION_PASSWORDS = "AWS Glue connection passwords MUST be configured for at-rest encryption"
    GLUE_IAM_PERMISSIONS = "AWS Glue MUST be configured to support IAM role based authentication"
    GLUE_CHECK_TAGS = "MUST be tagged in accordance with tagging standards"
    GLUE_JOB_CRAWLER_ENCRYPTION = "Data written by AWS Glue Job / Crawler MUST be configured for at-rest encryption"

class GlueMethodAssociations:
    GLUE_CHECK_DATA_CATALOG_ENCRYPTION = "check_data_catalog_encryption"
    GLUE_CHECK_CONNECTION_PASSWORDS = "check_connection_passwords"
    GLUE_IAM_PERMISSIONS = "check_iam_permissions"
    GLUE_CHECK_TAGS = "check_glue_tags"
    GLUE_JOB_CRAWLER_ENCRYPTION = "check_encryption_at_rest"

security_checks = {
    "GLUE_JOB_CRAWLER_ENCRYPTION": {
        "method_name": GlueMethodAssociations.GLUE_JOB_CRAWLER_ENCRYPTION,
        "check_description": GlueChecks.GLUE_JOB_CRAWLER_ENCRYPTION
    },
    "GLUE_CHECK_TAGS": {
        "method_name": GlueMethodAssociations.GLUE_CHECK_TAGS,
        "check_description": GlueChecks.GLUE_CHECK_TAGS
    }
}

account_security_checks = {
    "GLUE_CHECK_CONNECTION_PASSWORDS": {
        "method_name": GlueMethodAssociations.GLUE_CHECK_CONNECTION_PASSWORDS,
        "check_description": GlueChecks.GLUE_CHECK_CONNECTION_PASSWORDS
    },
    "GLUE_CHECK_DATA_CATALOG_ENCRYPTION": {
        "method_name": GlueMethodAssociations.GLUE_CHECK_DATA_CATALOG_ENCRYPTION,
        "check_description": GlueChecks.GLUE_CHECK_DATA_CATALOG_ENCRYPTION
    }
}

best_practices_checks = {
}

glue_dict = {
    "SecurityCheck": security_checks,
    "BestPractices": best_practices_checks,
    "AccountSecurityCheck": account_security_checks,
    "AccountBestPractices": {}
}

resource_arns = {
    'Catalog' : 'catalog',
    'Database':'database',
    'Table':'table',
    'Connection':'connection',
    'Crawler':'crawler',
    'Job':'job',
    'Trigger':'trigger',
    'DevEndPoint':'devEndpoint',
    'MachineLanguageTransform':'mlTransform',
}




import os
import boto3
import json
from botocore.client import Config
from jinja2 import Environment, FileSystemLoader, select_autoescape

from common.constants import application_constants as constants
from common.utils.initialize_logger import logger


def check_aws_credentials_exist():
    try:
        boto3.client('sts').get_caller_identity()
    except Exception as ex:
        raise ValueError("AWS credentials not configured.") from ex
    
def get_configuration():
    """
    Returns the Configuration like "AWS region name" and "retries" to use for the AWS client object.
    """
    try:
        
        region=os.environ.get('AWS_DEFAULT_REGION')
        config=Config(retries = { "max_attempts": constants.Generic.MAX_RETRIES },region_name=region)
        return config
    except Exception as ex:
        logger.error("Error occurred while setting configuration for the client:  %s", str(ex))
        raise ex


def is_least_privilege_sg(rules):
    """
    Check if the security group rules are the least privilege required.

    Args:
        rules (list): List of security group rules to check.

    Returns:
        bool: True if the security group rules are the least privilege required, else False.

    """
    for rule in rules:
        if rule.get('IpProtocol') == '-1' or rule.get('IpRanges') == [{'CidrIp': '0.0.0.0/0'}] or rule.get('Ipv6Ranges') == [{'CidrIpv6': '::/0'}]:
            return False
    return True


def custom_round(x, prec=2, base=0.01):
    """ Function to do custom round off
    Args:
        x (float): initial float value
        prec (int, optional): precision. Defaults to 0.
        base (float, optional): base value. Defaults to .04.
    Returns:
        int: custom round off value
    """
    return round(base * round(float(x)/base),prec)


def check_subnet_has_igw(subnet_id):
    """
    Check if the subnet associated with an EC2 instance has an Internet Gateway (IGW) in its route table.

    Args:
        ec2_instance_id (str): The ID of the EC2 instance.

    Returns:
        bool: True if the subnet associated with the instance has an IGW, False otherwise.

    Raises:
        Exception: If any error occurs during the check.
    """
    ec2_client = boto3.client('ec2')

    try:
        list_route_tables = ec2_client.describe_route_tables(
            Filters=[
                {
                    'Name': 'association.subnet-id',
                    'Values': [
                        subnet_id
                    ]
                }
            ]
        )
        if not list_route_tables['RouteTables']:
            subnet = ec2_client.describe_subnets(
                Filters=[
                    {
                        'Name': 'subnet-id',
                        'Values': [
                            subnet_id
                        ]
                    }
                ]
            )['Subnets'][0]
            route_table = ec2_client.describe_route_tables(
                Filters=[
                    {
                        'Name': 'vpc-id',
                        'Values': [
                            subnet['VpcId']
                        ]
                    },
                    {
                        'Name': 'association.main',
                        'Values': [
                           'true'
                        ]
                    }
                ]
            )['RouteTables'][0]
        else:
            route_table = list_route_tables['RouteTables'][0]
        for route in route_table['Routes']:
            if 'GatewayId' in route and route['GatewayId'].startswith('igw-'):
                return True

        return False

    except Exception as ex:
        logger.error("Error occurred while checking if subnet has Internet Gateway:  %s", str(ex))
        raise


def create_config_file(resource_names,config_changes):
    """
    Creates a new configuration file for AWS Services  with the specified resource names.

    Returns:
    - dict: The new configuration data.

    Raises:
    - ValueError: If the resource_names parameter is empty.
    """
    if not resource_names:
        raise ValueError("resource_names parameter cannot be empty")
    config_data = {}
    config_data["account_tags"] = []
    config_data["region_name"] = os.environ.get('AWS_DEFAULT_REGION')
    account_id = boto3.client('sts').get_caller_identity().get('Account')
    try:
        for resource in resource_names:
            resource_details = []

            for resource_id in resource_names[resource]:
                if account_id == resource_id:
                    security_checks = [{"check_name": check, "status": "Enabled"}
                                    for check in constants.overall_check_dict[resource][constants.Type.ACC_SECURITY_CHECK]]
                    best_practices = [{"check_name": check, "status": "Enabled"}
                                    for check in constants.overall_check_dict[resource][constants.Type.ACC_BEST_PRACTICES]]
                else:
                    security_checks = [{"check_name": check, "status": "Enabled"}
                                    for check in constants.overall_check_dict[resource][constants.Type.SECURITY_CHECK]]
                    best_practices = [{"check_name": check, "status": "Enabled"}
                                    for check in constants.overall_check_dict[resource][constants.Type.BEST_PRACTICES]]
                resource_details.append({
                    "resource_name": resource_id,
                    "security_requirement_checks": security_checks,
                    "security_best_practices_checks": best_practices
                })

            config_data[resource] = resource_details
        if config_changes:
            for key in config_changes:
                if  key in ('account_tags'):
                    config_data[key] = config_changes[key]
                elif len(config_changes[key])>0:
                    for resource in config_changes[key]:
                        for resource_new in config_data[key]:
                            if resource_new['resource_name']==resource['resource_name']:
                                for check in resource_new['security_requirement_checks']:
                                    if check['check_name']==resource['check_name']:
                                        check['status']=resource['status']
                                for check in resource_new['security_best_practices_checks']:
                                    if check['check_name']==resource['check_name']:
                                        check['status']=resource['status']
    except Exception as ex:
        logger.error("Error occurred when creating new configuration file: %s", str(ex))
        raise

    return config_data


def store_config_file(config_data):
    """
    Stores the specified configuration data to a file.

    Parameters:
    - config_data (dict): The configuration data to be stored.

    Returns:
    - None

    Raises:
    - Exception: If an error occurs while writing to the file.
    """
    try:
        filepath=os.environ.get('Config_Path')
        with open(filepath, 'w', encoding='utf-8') as outfile:
            json.dump(config_data, outfile, indent=4)
            logger.info("Created new configuration file.")

    except Exception as ex:
        logger.error("Error occurred when storing configuration file: %s", str(ex))
        raise


def generate_audit_report(service: dict, timestamp: str) -> None:
    """Generate audit report for a given service.

    Args:
        service (dict): A dictionary of check results for a service.
        timestamp (str): A timestamp string in the format YYYY-MM-DD_HH:MM:SS.

    Raises:
        Exception: If any error occurs while generating the audit report.

    Returns:
        None: Writes the html page to reports folder
    """
    logger.info("Generating audit report")
    try:
        for service_name, service_id in service.items():
            env = Environment(
                loader=FileSystemLoader(f'{os.getcwd()}/src/common/templates'),
                autoescape=select_autoescape(['html', 'xml'])
            )
            template = env.get_template('audit_report.html')
            html = template.render(check_results=service_id, service=service_name)

            service_name = service_name.lower()
            file_name = f"reports/{timestamp}/aws_services/{service_name}.html"
            os.makedirs(f"reports/{timestamp}/aws_services", exist_ok=True)

            with open(file_name, 'w') as f:
                f.write(html)

        index_result = {}
        for service_name, result in service.items():
            passed = 0
            failed = 0
            unknown = 0
            disabled = 0
            total = 0
            for key, value in result.items():
                for out in value:
                    if out['status'] == 'Passed':
                        passed += 1
                    elif out['status'] == 'Failed':
                        failed += 1
                    elif out['status'] == 'Disabled':
                        disabled += 1
                    elif out['status'] == 'Unknown':
                        unknown += 1
                    total += 1

            if total != 0:
                passed = (passed/total) * 100
                failed = (failed/total) * 100
                disabled = (disabled/total) * 100
                unknown = (unknown/total) * 100

            index_result[service_name] = {
                'Passed': custom_round(passed),
                'Failed': custom_round(failed),
                'Disabled': custom_round(disabled),
                'Unknown': custom_round(unknown)
            }

        # write index file
        index_template = env.get_template('index.html')
        index_html = index_template.render(service_title=index_result)
        index_file_name = f"reports/{timestamp}/index.html"

        with open(index_file_name, 'w') as f:
            f.write(index_html)

    except Exception as ex:
        logger.error("Error generating audit report: %s", str(ex))
        raise

    logger.info("Completed generating audit report")
